function [ y,x ] = generateDeclippingExp( fileName, iSNR, param )
%% generateDeclippingExp is the function to generate data for the declipping experiment
%
%   Input:
%       filename: string, clean input filename to process
%       iSNR: double, input SDR
%       param: parameters structure
%
%   Output:
%       y: clipped signal
%       x: original signal
%
% ************************************************************
% This software is available under the  BSD-3-Clause License  (https://opensource.org/licenses/BSD-3-Clause)
% ************************************************************
% 3-Clause BSD License
% 
% Copyright <2016-2019><INRIA - CNRS><Clément Gaultier, Srdan Kitic, Ewen Camberlein, Rémi Gribonval, Nancy Bertin>
% 
% Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
% 
% (i) Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
% 
% (ii) Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
% 
% (iii) Neither the name of the copyright holder nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.
% 
% THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
% 
% Contact: nancy.bertin@irisa.fr, frederic.bimbot@irisa.fr, remi.gribonval@inria.fr
% PANAMA Research Group,  IRISA  -  (Inria & CNRS)
% Campus de Beaulieu, 35042 Rennes cedex, France
%

outputDIR = ['./../../Results/Redundancy' num2str(param.redundancy) ...
    '/Declipping/' num2str(iSNR) '/'];
inpFile = [outputDIR fileName num2str(iSNR) 'dB_clip_input.wav'];
if exist(inpFile,'file')
    y = audioread(inpFile, 'double');
else
    cleanFile = ['./../../CleanAudioData/' fileName];
    x = audioread(cleanFile);
    y =  ClipSNR(x,iSNR);%Clipping(x,ClipLevel/100);%
end

end

%% Threshold for the given clipping level in dB
% Computes approximation of the maximum magnitude threshold to obtain given
% clipping SCDR value
%
function y = ClipSNR(x,SNRs)

nbChan = min(size(x)); 
nbSNR = length(SNRs);
if nbSNR == 1
    disp('Clipping all channels to the same SDR level')
    SNRs = SNRs*ones(1,nbChan);
elseif nbSNR == nbChan
    disp('Clipping each channel respectively with its own SDR value')
else
    error('Channel number and SDR clipping levels number must match')
end

for Chan = 1:nbChan
    
    % Difference function
    if SNRs(Chan)==0
        cSNR=@(T)10*log10(sum(x(:,Chan).^2)/sum((x(:,Chan)-Clipping(x(:,Chan),T)).^2))-SNRs(Chan)-0.1; % safety for the fzero step below
    else
        cSNR=@(T)10*log10(sum(x(:,Chan).^2)/sum((x(:,Chan)-Clipping(x(:,Chan),T)).^2))-SNRs(Chan);
    end
    
    % Search range for fzero function
    Upper=0.99*max(abs(x(:,Chan)));
    Lower=eps;
    % Minimization of the difference function
   
    [T, True_SNR]=fzero(cSNR,[Lower Upper]);
  
   
    % True SCDR will most probably be slightly different than proposed SCDR
    True_SNR=True_SNR+SNRs(Chan);
    
    % Output the acutual clipped data
    y(:,Chan) = Clipping(x(:,Chan),T);
    
end

end

%% Clipping
% Clips the signal x to the threshold T and returns the output signal and
% indices of reliable and clipped samples
%
% 
function y=Clipping(x,T)

y=x;
ind = (abs(x)>T);
y(ind) = sign(x(ind)) .* T;

end