function [ y,x,noiseVar ] = generateDenoisingExp( fileName, iSNR, param)
%% generateDenoisingExp is the function to generate data for the denoising experiment
%
%   Input:
%       filename: string, clean input filename to process
%       iSNR: double, input SNR
%       param: parameters structure
%
%   Output:
%       y: noisy signal
%       x: original signal
%       noiseVar: noise variance
%
% ************************************************************
% This software is available under the  BSD-3-Clause License  (https://opensource.org/licenses/BSD-3-Clause)
% ************************************************************
% 3-Clause BSD License
% 
% Copyright <2016-2019><INRIA - CNRS><Clément Gaultier, Srdan Kitic, Ewen Camberlein, Rémi Gribonval, Nancy Bertin>
% 
% Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
% 
% (i) Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
% 
% (ii) Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
% 
% (iii) Neither the name of the copyright holder nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.
% 
% THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
% 
% Contact: nancy.bertin@irisa.fr, frederic.bimbot@irisa.fr, remi.gribonval@inria.fr
% PANAMA Research Group,  IRISA  -  (Inria & CNRS)
% Campus de Beaulieu, 35042 Rennes cedex, France
%


outputDIR = ['./../../Results/Redundancy' num2str(param.redundancy) ...
    '/Denoising/' num2str(iSNR) '/'];
inpFile = [outputDIR fileName num2str(iSNR) 'dB_noisy_input.wav'];
if exist(inpFile,'file')
    y = audioread(inpFile, 'double');
else
    cleanFile = ['./../../CleanAudioData/' fileName];
    x = audioread(cleanFile);
    NoiseParams.Predict = true;
    NoiseParams.Type = 'WHITE';
    y = Noisegen(x,iSNR,NoiseParams);
end
noiseVar = var(x-y);
noiseMagn = std(x-y); % sqrt(noise_var)

end

function [ NoisySignal ] = Noisegen( CleanSignal, SNR, NoiseParams )
%Noisegen Outputs the CleanSignal with additive noise defined by params in
%NoiseParams
%   CleanSignal : vector (double)
%   SNR : the desired SNR 
%   NoiseParams : Struct containing all the parameters for the noise to add
%   NoisySignal : output noisy signal
%


if nargin < 3
    NoiseParams = [];
end

if ~isfield(NoiseParams, 'Mean') || isempty(NoiseParams.Mean)
    NoiseParams.Mean = 0;
end

if ~isfield(NoiseParams, 'Variance') || isempty(NoiseParams.Variance)
    NoiseParams.Variance = 1;
end

if ~isfield(NoiseParams, 'Skewness') || isempty(NoiseParams.Skewness)
    NoiseParams.Skewness = 0;
end

if ~isfield(NoiseParams, 'Kurtosis') || isempty(NoiseParams.Kurtosis)
    NoiseParams.Kurtosis = 3;
end


if ~isfield(NoiseParams, 'Type') || isempty(NoiseParams.Type)
    NoiseParams.Type = 'WHITE';
end

if ~isfield(NoiseParams, 'Predict') || isempty(NoiseParams.Predict)
    NoiseParams.Predict = true;
end

if NoiseParams.Predict
    rng('default');
    rng(1);
else
    rng('shuffle');
end

NoiseLevel = getLeveldBFS(CleanSignal) - SNR;
L = length(CleanSignal);

Noise = pearsrnd(NoiseParams.Mean,sqrt(NoiseParams.Variance),...
        NoiseParams.Skewness,NoiseParams.Kurtosis,L,1);
    
switch upper(NoiseParams.Type)
    case 'WHITE'
       % no filtering
    case 'PINK'
        B = [0.049922035 -0.095993537 0.050612699 -0.004408786];
        A = [1 -2.494956002   2.017265875  -0.522189400];
        Noise = filter(B,A,Noise); % Apply 1/F roll-off to PSD
        
    case 'LTAS' % long term average spectrum (to be used with speech)
        [Coeff, Var] = LTASFilter(CleanSignal,6);
        Noise = filter(Var,Coeff,Noise);
    otherwise
        disp('Unknown noise condition!');
end


Noise = setLeveldBFS(Noise,NoiseLevel);

NoisySignal = Noise + CleanSignal;

end

function [ coeff, variance ] = LTASFilter( InputSig, order )
%LTASSFilter provide the coefficients corresponding to the linear
%predicted long term average InputSig spectrum filter (for generating
%speech shaped noise for example)
%   InputSig : vector (double) signal from which to design the filter
%   order : int order for the linear predictive esstimation
%   coeff,variance : filter coefficients and variance [a,g]
%

data = InputSig/sqrt((sum(InputSig.^2))/length(InputSig));
[coeff,variance]= lpc(data,order);

end


