function [ wHat, proxParam ] = proxL2distind( b, y, A, proxParam, prior)
%% proxL2distind Solves the following optimization problems:
% Either for synthesis prior:
%   wHat = argmin_w||w-b||^2 s.t. ||A'w - y||<=epsilon
%   works if A is a parseval tight frame
%
% or for analysis prior:
%   wHat = argmin_w||w-A'b||^2 s.t. ||w - y||<=epsilon
%
%   Input:
%       b: estimation
%       y: initial input signal
%       A: function handle to the frequency transform dictionnary/analysis
%          operator
%       proxParam: parameter structure with epsilon field
%       prior: (string) 'analysis' or 'synthesis'
%
%   Output:
%       wHat: estimated signal projected on the l2 constraint
%       proxParam: updated parameter structure
%
% ************************************************************
% This software is available under the  BSD-3-Clause License  (https://opensource.org/licenses/BSD-3-Clause)
% ************************************************************
% 3-Clause BSD License
% 
% Copyright <2016-2019><INRIA - CNRS><Clément Gaultier, Srdan Kitic, Ewen Camberlein, Rémi Gribonval, Nancy Bertin>
% 
% Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
% 
% (i) Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
% 
% (ii) Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
% 
% (iii) Neither the name of the copyright holder nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.
% 
% THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
%
% Contact: nancy.bertin@irisa.fr, frederic.bimbot@irisa.fr, remi.gribonval@inria.fr
% PANAMA Research Group,  IRISA  -  (Inria & CNRS)
% Campus de Beaulieu, 35042 Rennes cedex, France

epsilon = proxParam.epsilon;

if strcmpi(prior,'analysis')

    xHat = A(b,'transp');
    d = xHat - y;
    norm_d = norm(d,'fro');

    if norm_d > epsilon
        wHat = xHat - ((norm_d - epsilon)/norm_d) .* d;
    else
        wHat = xHat;
    end

else%synthesis

    d = A(b,'transp') - y;
    norm_Hb = norm(d,'fro');

    if norm_Hb > epsilon
        wHat = b - ((norm_Hb - epsilon)/norm_Hb) .* A(d,'notransp');
    else
        wHat = b;
    end


end
