function [ Stencils ] = generateTFPatterns( Setup, param )
%% generateTFPatterns Generates useful time-frequency patterns used for the
%(analysis/synthesis) social denoising or declipping
%
%   Input:
%       Setup: char indicating if the required configuration is 'minimal', 
%              'default' or 'extended' (2, 5 or 6 available stencils)
%       param: struct with other parameters
%           param.redundancy: redundancy factor of the frequency transform used
%                         (controls the spread over frequency)
%           param.timeBlocks: time spread of the stencil
%
%   Output:
%       Stencils: TF binary masks
%
% ************************************************************
% This software is available under the  BSD-3-Clause License  (https://opensource.org/licenses/BSD-3-Clause)
% ************************************************************
% 3-Clause BSD License
% 
% Copyright <2016-2019><INRIA - CNRS><Clément Gaultier, Srdan Kitic, Ewen Camberlein, Rémi Gribonval, Nancy Bertin>
% 
% Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
% 
% (i) Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
% 
% (ii) Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
% 
% (iii) Neither the name of the copyright holder nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.
% 
% THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
% 
% Contact: nancy.bertin@irisa.fr, frederic.bimbot@irisa.fr, remi.gribonval@inria.fr
% PANAMA Research Group,  IRISA  -  (Inria & CNRS)
% Campus de Beaulieu, 35042 Rennes cedex, France
%


if ~isfield(param,'redundancy') || isempty(param.redundancy)
    param.redundancy = 1;
end

if ~isfield(param,'timeBlocks') || isempty(param.timeBlocks)
    param.timeBlocks = 5;
end


T = param.timeBlocks;
R = param.redundancy;


switch upper(Setup)
    case 'MINIMAL' % minimal set of stencils (2)
        Stencils{1} = TonalTFStencil(T,R);
        Stencils{2} = PreEchoTFStencil(T,R);
        
    case 'DEFAULT' % default set of stencils (5)
        Stencils{1} = TonalTFStencil(T,R);
        Stencils{2} = PreEchoTFStencil(T,R);
        Stencils{3} = TransientTFStencil(T,R);
        Stencils{4} = TonalupTFStencil(T,R);
        Stencils{5} = TonaldownTFStencil(T,R);
        
    case 'EXTENDED' % extended set of stencils (6)
        Stencils{1} = TonalTFStencil(T,R);
        Stencils{2} = PreEchoTFStencil(T,R);
        Stencils{3} = TransientTFStencil(T,R);
        Stencils{4} = TonalupTFStencil(T,R);
        Stencils{5} = TonaldownTFStencil(T,R);
        Stencils{6} = BlobTFStencil(T,R);
        
    otherwise
        error('Unknown time-frequency stencil setup')
end

%% functions for building the desired TF patterns

% The created binary patterns have T columns and F = R*T rows
% i.e. for redundancy 1, binary masks are square matrices 

    function [TonalWeights] = TonalTFStencil(T,R)
        TonalWeights = [zeros(2*R,T);ones(R,T);zeros(2*R,T)];
    end

    function [TransientWeights] = TransientTFStencil(T,R)
        TransientWeights = [zeros(R*T,(T-1)/2) ones(R*T,1) zeros(R*T,(T-1)/2)];
    end

    function [TonalupWeights] = TonalupTFStencil(T,R)
        if R ~=1
            Block = mat2cell(ones(R,T),[R],[ones(1,T)]);
            TonaldownWeights = blkdiag(Block{:});
            TonalupWeights = flipud(TonaldownWeights);
        else
            TonalupWeights = flipud(eye(T));
        end
    end

    function [TonaldownWeights] = TonaldownTFStencil(T,R)
        if R ~=1
            Block = mat2cell(ones(R,T),[R],[ones(1,T)]);
            TonaldownWeights = blkdiag(Block{:});
        else
            TonaldownWeights = eye(T);
        end
    end

    function [PreEchoWeights] = PreEchoTFStencil(T,R)
        PreEchoWeights = [zeros(R*T,(T-1)/2) ...
            [zeros(R,1);ones(R*(T-2),1);zeros(R,1)] ...
            [zeros(R*2,(T-1)/2);ones(R,(T-1)/2);zeros(R*2,(T-1)/2)]];
    end

    function [BlobWeights] = BlobTFStencil(T,R)
        BlobWeights = [zeros(R*T,1) [zeros(1,T-2);ones(R*T-2,T-2);zeros(1,T-2)] zeros(R*T,1)];
    end





end

