function s = shrinkHardThresholding_DFT(z, k, shrinkParams)

%% shrinkHardThresholding_FFT: the Hard Thresholding operator
%   (fast and adapted to the symmetry in z due to DFT cplx transforms)
%
%   Input:
%       z: the vector to shrink
%       k: the number of coefficients to keep in z
%
%   Output:
%       s: the thresholded vector
%
% ************************************************************
% This software is available under the  BSD-3-Clause License  (https://opensource.org/licenses/BSD-3-Clause)
% ************************************************************
% 3-Clause BSD License
% 
% Copyright <2016-2019><INRIA - CNRS><Clément Gaultier, Srdan Kitic, Ewen Camberlein, Rémi Gribonval, Nancy Bertin>
% 
% Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
% 
% (i) Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
% 
% (ii) Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
% 
% (iii) Neither the name of the copyright holder nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.
% 
% THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
% 
% Contact: nancy.bertin@irisa.fr, frederic.bimbot@irisa.fr, remi.gribonval@inria.fr
% PANAMA Research Group,  IRISA  -  (Inria & CNRS)
% Campus de Beaulieu, 35042 Rennes cedex, France
%

N = length(z); % FFT Size
s = zeros(N,1);

if (mod(k,2)~=0) % k must be even
    error('[shrinkHardThresholding_DFT]: k must be even !');
end

if (k < N)
    
    a_abs = abs(z(2:(N/2)));
    [~, indSorted] = sort(a_abs, 'descend');
    indSorted = indSorted + 1; % as we postpone taking into account F=0 component
    
    bF_0       = 0; % F=0
    bF_Nyquist = 0; % F=Fs/2
    
    % Set flags for F=0 and Nyquist frequencies if > min sorted value of
    % interest (index k/2 in indSorted)
    if abs(z(1)) > abs(z(indSorted(k/2))) % F=0 component > min sorted value of interest
        bF_0 = 1;
    end
    
    if abs(z(N/2+1)) > abs(z(indSorted(k/2))) % F=Fs/2 component > min sorted value of interest
        bF_Nyquist = 1;
    end
    
    % choice of index to be kept depending of previous flags
    if (bF_0 || bF_Nyquist)
        
        % remove one couple of values (min of sorted ones) to add new ones (F=0 and/or F=Fs/2)
        ind = [indSorted(1:k/2 -1);(N-indSorted(1:k/2 -1))+2]; 
        
        if(bF_0)
            ind =[ind;1];
        end
        
        if (bF_Nyquist)
            ind =[ind;N/2+1];
        end
        
    else 
        
        % we keep the K/2 first components as well as symetric part of the
        % spectrum (=> K components)
        ind = [indSorted(1:k/2);(N-indSorted(1:k/2))+2];
        
    end
    
    s(ind) = z(ind);

else
    s = z; % we keep all components
end


end
