function [S] = shrinkPEW(Z, lambda, shrinkParam)
%% PEW: Persistent Empirical Wiener operator
%
%   Input:
%       Z: input matrix to shrink
%       lambda: skrinking parameter
%       proxParam: a structure containing:
%          prev_blocks (optionnal) previoulsy processed data to account for
%          for the shrinkage operation
%          W time-frequency pattern
%          isRedundant boolean flag to account for redundant cases (tall Z)
%
%   Output:
%       S: thresholded matrix
%
%   References:
%       [1] M. Kowalski, K. Siedenburg, and M. Dorfler, Social sparsity! neighborhood systems enrich structured shrinkage operators, IEEE Transactions on Signal
%       Processing, vol. 61, no. 10, pp. 2498-2511, 2013.
%       [2] M. Kowalski, Thresholding rules and iterative shrinkage/thresholding algorithm:
%       A convergence study, in IEEE International Conference on Image Processing
%       (ICIP). IEEE, 2014, pp. 4151-4155.
%
% ************************************************************
% This software is available under the  BSD-3-Clause License  (https://opensource.org/licenses/BSD-3-Clause)
% ************************************************************
% 3-Clause BSD License
% 
% Copyright <2016-2019><INRIA - CNRS><Clément Gaultier, Srdan Kitic, Ewen Camberlein, Rémi Gribonval, Nancy Bertin>
% 
% Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
% 
% (i) Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
% 
% (ii) Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
% 
% (iii) Neither the name of the copyright holder nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.
% 
% THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
% 
% Contact: nancy.bertin@irisa.fr, frederic.bimbot@irisa.fr, remi.gribonval@inria.fr
% PANAMA Research Group,  IRISA  -  (Inria & CNRS)
% Campus de Beaulieu, 35042 Rennes cedex, France
%

isRedundant = shrinkParam.isRedundant;
prev_blocks = shrinkParam.prev_blocks;
W = shrinkParam.W;


if ~isRedundant
    bins = size(Z,1);
blocks = size(Z,2);

if isempty(prev_blocks)
    startblock = 1;
else
    startblock = blocks;
end

if isempty(W)
    W = 1;
end

widthW = (size(W,2) - 1)/2;
heigthW = (size(W,1) - 1)/2;

if blocks > 1   
    B = [abs(prev_blocks).^2 abs(Z).^2];
    
    B = [fliplr(B(:,2:2+widthW-1)) B fliplr(B(:,end-widthW : end-1))];
    B = [flipud(B(2:2+heigthW-1,:)); B; flipud(B(end-heigthW:end-1,:))];

    Ztilde = conv2(B,W,'valid') / norm(W,1);
    
    Ztilde = Ztilde(:, startblock:end);
    
else
    Ztilde = abs(Z).^2;
end

S = Z.*max(0, 1-lambda^2./Ztilde);

else
    bins = size(Z,1);
blocks = size(Z,2);

if isempty(prev_blocks)
    startblock = 1;
else
    startblock = blocks;
end

if isempty(W)
    W = 1;
end

widthW = floor((size(W,2) - 1)/2);
heigthW = floor((size(W,1) - 1)/2);

if blocks > 1   
    B = [abs(prev_blocks).^2 abs(Z).^2];
    % mirroring on borders
    B = [fliplr(B(:,2:2+widthW-1)) B fliplr(B(:,end-widthW : end-1))];
    B = [flipud(B(1:heigthW,:)); B; flipud(B(end-heigthW+1:end,:))];

    Ztilde = conv2(B,W,'same') / norm(W,1);
    
    if isempty(prev_blocks)
        Ztilde = Ztilde(heigthW+1:end-heigthW, widthW+1:end-widthW);
    else
        Ztilde = Ztilde(heigthW+1:end-heigthW, startblock+widthW:end-widthW);
    end
    
else
    Ztilde = abs(Z).^2;
end

S = Z.*max(0, 1-lambda^2./Ztilde);
end



end