function [RestoredData, param, Perf] = enhanceAllFrames(y,x,param)
%% enhanceAllFrames is the wrapper function to process all the signal
% frame by frame processing with the sparse audio reconstruction
%
%   Input:
%       y: the input degraded signal (1D vector)
%       x: the original clean signal for ground truth comparison
%       param: parameters structure
%
%   Output:
%       RestoredData: the enhanced signal (1D vector)
%       param: the updated parameters structure
%       Perf: a structure containing log informations (time, improvement...)
%
% ************************************************************
% This software is available under the  BSD-3-Clause License  (https://opensource.org/licenses/BSD-3-Clause)
% ************************************************************
% 3-Clause BSD License
% 
% Copyright <2016-2019><INRIA - CNRS><Clément Gaultier, Srdan Kitic, Ewen Camberlein, Rémi Gribonval, Nancy Bertin>
% 
% Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
% 
% (i) Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
% 
% (ii) Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
% 
% (iii) Neither the name of the copyright holder nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.
% 
% THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
% 
% Contact: nancy.bertin@irisa.fr, frederic.bimbot@irisa.fr, remi.gribonval@inria.fr
% PANAMA Research Group,  IRISA  -  (Inria & CNRS)
% Campus de Beaulieu, 35042 Rennes cedex, France
%
%% Checking and setting parameters

if nargin < 3
    param = [];
end
param = checkAndInitGlobalParams(param);

% Setting the shrinkage operator
switch upper(param.Shrinkage)
    case 'PEW'
        param.rate = 1;
        param.timeBlocks = 5;
        param.lowiter = 10;
        param.Gamma = generateTFPatterns('Extended',param);
        param.nb_blocks = size(param.Gamma{1},2);
        k_start = param.nb_blocks*param.stepsize + 1;
        shrink = @(Z,lambda,shrinkParam)shrinkPEW(Z,lambda,shrinkParam);
        updateSparsity = @(k,relax)geometricUpdate(k,relax); % k(i+1) = relax*k(i)
    case 'HT'
        param.rate = 1;
        param.timeBlocks = 0;
        param.nb_blocks = 0;
        k_start =  1;
        updateSparsity = @(k,relax)arithmeticUpdate(k,relax); % k(i+1) = k(i)+relax
    otherwise
        error('Unknown shrinkage!')
end

% Setting the time-frequency transform and shrinkage parameters
switch upper(param.transform)
    case 'DFT'
        A = @(inp,transp)overcompleteDFT(inp,transp,param.redundancy);
        if strcmpi(param.Shrinkage,'HT')
            shrink = @(z,k,shrinkParam)shrinkHardThresholding_DFT(z,k,shrinkParam);
            param.greedy = 2;
            param.relax = 2;
            param.iter = param.w_size*param.redundancy/param.relax + 1;
        end
    case 'DCT'
        A = @(inp,transp)overcompleteDCT(inp,transp,param.redundancy);
        if strcmpi(param.Shrinkage,'HT')
            shrink = @(z,k,shrinkParam)shrinkHardThresholding(z,k,shrinkParam);
            param.greedy = 1;
            param.relax = 1;
            param.iter = param.w_size*param.redundancy/param.relax + 1;
        end
    otherwise
        error('Unknown transform!')
end

% Setting the projection operator
switch upper(param.Pb)
    case 'DENOISING'
        prox = @(b,y,H,proxParam)proxL2distind(b,y,H,proxParam,param.prior);
    case 'DECLIPPING'
        prox = @(b,y,H,proxParam)proxLinfClip(b,y,H,proxParam,param.prior);
    otherwise
        error('Unknown audio reconstruction problem!')
end

noise_var = var(y-x);
noise_magn = std(y-x);

% zero padding
param.blockSize = 2*param.nb_blocks*param.stepsize + param.w_size;
stepsize = param.stepsize;
[y,OrigInd] = performZeroPadding(y,param.w_size,stepsize,param.timeBlocks);
[x,~] =performZeroPadding(x,param.w_size,stepsize,param.timeBlocks);

% initial and static data
if param.redundancy > 1
    algParam.isRedundant = true;
else
    algParam.isRedundant = false;
end
t = param.redundancy * param.w_size;
algParam.shrinkage = param.Shrinkage;
algParam.prior = param.prior;
algParam.Pb = param.Pb;
algParam.crime = false;
algParam.greedy = param.greedy;
algParam.rate = param.rate;
algParam.loud = 0;
algParam.iter = param.iter;
algParam.accuracy = param.accuracy;
algParam.weights = {};
algParam.underrelax = param.underrelax;
algParam.z_prev = zeros(t,param.nb_blocks);
algParam.z_post = zeros(t,param.nb_blocks);
disp(['Shrinkage: ' algParam.shrinkage])
wa=sqrt(param.window(param.w_size));
ws=wa;
y_length = length(y);
yMax = max(abs(y));
if strcmpi(param.Pb,'DECLIPPING')
    clipped = (abs(y) == yMax);
    clipPos = (clipped & y > 0);
    clipNeg = (clipped & y < 0);
else
    clipped = false(length(y),1);
    clipPos = clipped;
    clipNeg = clipped;
end
reliable = ~clipped;
n=0;
Relax = [];
Regularizer = [];
IterCount = [];
Undersampling = [];
Improvement = [];
Stencil = [];
Reconst=zeros(param.w_size,2);
Estimate=zeros(param.w_size,2);
algParam.residual_old = [];

if param.loud
    h = waitbar(0, [param.Pb '... '],'Name',param.Pb,...
            'CreateCancelBtn',...
            'setappdata(gcbf,''canceling'',1)');
    waitObject = onCleanup(@() delete(h));
    setappdata(h,'canceling',0)
end

if param.causal
    blocks = param.nb_blocks + 1;
else
    blocks = 2*param.nb_blocks + 1;
end

k_end =  y_length - param.blockSize;
globalTimerStart = tic;
tElapsed = 0;
%% Framing the signal
for k = k_start : stepsize : k_end
    n=n+1;
    switch upper(param.Shrinkage)
        case 'PEW'
        if n == 1 && param.causal == true
            midblock = 1;
            prev_blocks = zeros(t,param.nb_blocks);
            for bl = 1 : param.nb_blocks
                prev_blocks(:,bl) = A(y(bl*stepsize : bl*stepsize + param.w_size-1) .* wa, 'notransp');
            end
            yChunk = zeros(param.w_size,blocks);
            yChunkNoWin = yChunk;
            relChunk = false(param.w_size,blocks);
            total_window = repmat(param.window(param.w_size),1,blocks);
            bl_start = 0;
        elseif param.causal == false
            midblock = param.nb_blocks + 1;
            prev_blocks = [];
            yChunk = zeros(param.w_size,blocks);
            yChunkNoWin = yChunk;
            relChunk = false(param.w_size,blocks);
            for bl = param.nb_blocks :-1 : 1
                yChunkNoWin(:,param.nb_blocks - bl + 1) = y(k - bl*stepsize : k - bl*stepsize + param.w_size-1);
                yChunk(:,param.nb_blocks - bl + 1) = yChunkNoWin(:,param.nb_blocks - bl + 1) .* wa;
                relChunk(:,param.nb_blocks - bl + 1) = reliable(k - bl*stepsize : k - bl*stepsize + param.w_size-1);
                clipPosChunk(:,param.nb_blocks - bl + 1) = clipPos(k - bl*stepsize : k - bl*stepsize + param.w_size-1);
                clipNegChunk(:,param.nb_blocks - bl + 1) = clipNeg(k - bl*stepsize : k - bl*stepsize + param.w_size-1);
            end
            bl_start = param.nb_blocks;
            total_window = repmat(param.window(param.w_size),1,blocks);
        end
        for bl = 1 : param.nb_blocks + 1
            yChunkNoWin(:,bl_start+bl) = y(k + (bl-1)*stepsize : k + (bl-1)*stepsize + param.w_size-1);
            yChunk(:,bl_start+bl) = yChunkNoWin(:,bl_start+bl) .* wa;
            relChunk(:,bl_start+bl) = reliable(k + (bl-1)*stepsize : k+(bl-1)*stepsize + param.w_size-1);
            clipPosChunk(:,bl_start+bl) = clipPos(k + (bl-1)*stepsize : k+(bl-1)*stepsize + param.w_size-1);
            clipNegChunk(:,bl_start+bl) = clipNeg(k + (bl-1)*stepsize : k+(bl-1)*stepsize + param.w_size-1);
        end
        clipChunk = ~relChunk;
        case 'HT'
            yChunkNoWin=y(k:k+param.w_size-1);
            yChunk = yChunkNoWin .* wa;
            relChunk = reliable(k:k+param.w_size-1);
            clipChunk = ~relChunk;
            clipPosChunk = clipPos(k:k+param.w_size-1);
            clipNegChunk = clipNeg(k:k+param.w_size-1);
            midblock = 1;
            prev_blocks = [];
    end
        
    % Current input frame    
    % Indexing reliable and clipped samples for the frame k
    algParam.midblock = midblock;
    
    
    % Ground truth for the evaluation
    if exist('x','var')
        True_f=x(k:k+param.w_size - 1) .* wa;
    end
    
    mn = nnz(relChunk(:,midblock))/numel(relChunk(:,midblock));
    
    %% Processing
    
if (nnz(clipChunk(:,midblock))<2 && strcmpi(param.Pb,'DECLIPPING'))
        xHat = yChunk;
        y_recovered = yChunk(:,midblock);
        z_recovered = A(y_recovered,'notransp');
        L = NaN;
        mn = NaN;
        clip_ratio = NaN;
        improve = NaN;
        algParam.relax = NaN;
        algParam.regularizer = NaN;
        algParam.iterCount = 0;
        best_w = 0;
        clear x0
        
else
        algParam.True_f = True_f;
        
        algParam.shrinkage = param.Shrinkage;

        if (exist('xOld','var') && strcmpi(param.Shrinkage,'PEW'))
           x0 = [xOld(:,2:end) yChunk(:,end)];
        else
           x0 = yChunk;
        end

        % initial point
        if strcmpi(algParam.prior,'analysis')
            algParam.x0 = x0;
        else
            algParam.x0 = A(x0,'notransp');
        end
        
        max_y = max(abs(yChunkNoWin(:)));
        algParam.time = 0;
        
        switch upper(param.Shrinkage)
            case 'HT'
                % Noise energy as a function of the applied window
                algParam.noise = sqrt(sum(wa(relChunk).^2 .* noise_var));
                algParam.relax = param.relax;
            case 'PEW'
                algParam.iter = param.lowiter;
                if strcmpi(param.Pb,'DECLIPPING')
                    algParam.relax = 0.99;
                else
                    % Relaxation parameter (inverse SNR)
                    algParam.relax = min(0.99,sqrt(noise_var/var(yChunkNoWin(:))));
                    algParam.noise = sqrt(sum(total_window(relChunk).^2 .* noise_var));
                end
                %% Adaptive stencil
                algParam.Gamma = param.Gamma;
                [best_w, best_greedy, xBest, algParam] = selectTFPattern( yChunk, A, relChunk, prev_blocks, max_y, prox, shrink, updateSparsity, algParam );
                algParam.weights = param.Gamma{best_w};
                algParam.greedy = best_greedy;
                % init point
                if strcmpi(param.prior,'analysis')
                    algParam.x0 = xBest;
                else
                    algParam.x0 = real(A(xBest,'notransp'));
                end
            otherwise
                error('Undefined audio restoration problem!');
        end
             
        algParam.crime = false;
        algParam.iter = param.iter;
        algParam.time = 0;
        
        [xHat,zHat,algParam] = generalTightSPADE(yChunk, A, prox, shrink, updateSparsity, relChunk, prev_blocks,algParam);
        
        if strcmpi(param.Pb,'DENOISING')
            %% Wiener filtering
            % Central
          %win_scale = sum(total_window(relChunk).^2) / nnz(relChunk);
          win_scale = 1; %  This scaling seems inapproriate but works better
          
          y_recovered = performWienerFiltering(xHat(:,midblock), ...
             yChunk(:,midblock), win_scale*noise_magn);          
         
        else
           y_recovered = xHat(:,midblock);
        end        
               
        algParam.residual_old = abs(A(yChunk - xHat,'notransp'));
        
        xOld = xHat;
        
        % Recovered signal
        z_recovered = A(y_recovered,'notransp');
        %tElapsed = tElapsed + algParam.time;
        % Some output
        improve = getSDR(True_f,y_recovered) - getSDR(True_f,yChunk(:,midblock));
        
        
end
    
    
    
    
        Relax = [Relax; algParam.relax];
        Improvement = [Improvement; improve];
        if strcmpi(param.Shrinkage,'PEW')
            Stencil = [Stencil; best_w];
        end
        Regularizer = [Regularizer; algParam.regularizer];
        IterCount = [IterCount; algParam.iterCount];
    
    if param.loud
        switch upper(param.Shrinkage)
            case 'PEW'
                logMessage = ['Block improvement = ' ...
                    num2str(round(improve,2)) 'dB (stencil = ' num2str(best_w) ...
                    ', lambda = ' num2str(Regularizer(end)) ...
                    ', alpha = ' num2str(round(algParam.relax,4)) ')'];
            case 'HT'
                logMessage = ['Improvement = ' ...
                    num2str(round(improve,2)) 'dB'];
        end
        
        if getappdata(h,'canceling'); delete(h); error('Canceled by user.'); end
        waitbar(k/k_end,h,'Processing... ');
    end
        
    prev_blocks = [prev_blocks(:,2:end) z_recovered];
        
    
    % Preserve estimated data
    Reconst(:,n)=y_recovered;
    Estimate(:,n)=xHat(:,midblock);
    % Preserve frame position
    RecPos(n)=k;
    Undersampling = [Undersampling; mn];
end %k = 1 : stepsize : k_end

%% Overlap and add
ReconstSignal_full = zeros(y_length,1);
NormWin=zeros(y_length,1);

for k=1:n
    OverlapAdd=Reconst(:,k) .* ws;
    ReconstSignal_full(RecPos(k):RecPos(k)+param.w_size-1) = ...
        ReconstSignal_full(RecPos(k):RecPos(k)+param.w_size-1) ...
        + OverlapAdd;
    
    NormWin(RecPos(k):RecPos(k)+param.w_size-1) = ...
        NormWin(RecPos(k):RecPos(k)+param.w_size-1) + ws(:).*wa(:);
end
ReconstSignal_full=ReconstSignal_full./NormWin;
tElapsed = toc(globalTimerStart);

%% Trimming signals to match lengths
start_seq=OrigInd(1);
end_seq=OrigInd(2);
RestoredData = ReconstSignal_full(start_seq:end_seq);

% Logging performances
Perf.Relax = Relax;
Perf.Regularizer = Regularizer;
Perf.Iter = IterCount;
Perf.Undersampling = Undersampling;
Perf.Improvement = Improvement;
Perf.TimeElapsed = tElapsed;
if strcmpi(param.Shrinkage,'PEW')
    Perf.Stencil = Stencil;
end

if param.loud
     delete(h)
end

end
