function [ xOut,zOut,varargout ] = generalTightSPADE( y, A, prox, ...
                                    shrink, updateSparsity, relChunk, prev_blocks, param )
%% generalTightSPADE is the core algorithm for the sparse audio frame based reconstruction
%
%   Input:
%       y: the input degraded signal (vector or matrix if social
%          modeling is used)
%       A: function handles on forward or inverse frequency transforms
%       prox: function handle on the projection to data-fidelity constraint
%       shrink: function handle on the sparsifying operator used
%       updateSparsity: function handle on the update rule for the sparsity
%                       hyperparameter (for now arithmetic or geometric)
%       relChunk: logical vector or matrix counting the reliables samples
%                 in yChunk (useful for declipping)
%       prev_blocks: used in case of accelerated social sparse modeling
%       param: parameters structure
%
%   Output:
%       xOut: time-domain audio estimate
%       zOut: frequency-domain estimate
%       varargout: cell table with optionnal outputs
%           varargout{1}: param (updated parameters structure)
%
% ************************************************************
% This software is available under the  BSD-3-Clause License  (https://opensource.org/licenses/BSD-3-Clause)
% ************************************************************
% 3-Clause BSD License
% 
% Copyright <2016-2019><INRIA - CNRS><Clément Gaultier, Srdan Kitic, Ewen Camberlein, Rémi Gribonval, Nancy Bertin>
% 
% Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
% 
% (i) Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
% 
% (ii) Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
% 
% (iii) Neither the name of the copyright holder nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.
% 
% THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
% 
% Contact: nancy.bertin@irisa.fr, frederic.bimbot@irisa.fr, remi.gribonval@inria.fr
% PANAMA Research Group,  IRISA  -  (Inria & CNRS)
% Campus de Beaulieu, 35042 Rennes cedex, France
%

param.objective = [];
param.k = [];
param.iterCount = [];


if ~isfield(param, 'prior') || isempty(param.prior)
    % analysis or synthesis sparse prior
    param.prior = 'analysis';
end

if ~isfield(param, 'isRedundant') || isempty(param.isRedundant)
    param.isredundant = false;
end

if ~isfield(param, 'loud') || isempty(param.loud)
    % Verbose
    param.loud = false;
end

if ~isfield(param, 'accuracy') || isempty(param.accuracy)
    % Accuracy threshold
    param.accuracy = 1e-1;
end

if ~isfield(param, 'x0') || isempty(param.x0)
    % Initial point
    initPoint = 'not provided.';
    if(strcmpi(param.prior,'analysis'))
        param.x0 = y;
    else
        param.x0 = A(y,'notransp');
    end
    
else
    initPoint = 'provided.';
end

if ~isfield(param, 'rate') || isempty(param.rate)
    % Relaxation rate
    param.rate = 1;
end

if ~isfield(param, 'greedy') || isempty(param.greedy)
    % Relaxation stepsize or the targeted sparsity level
    param.greedy = 1;
end

if ~isfield(param, 'crime') || isempty(param.crime)
    % Sparsity level known?
    param.crime = false;
end

if ~isfield(param, 'noise') || isempty(param.noise)
    % Noise level
    param.noise = 0;
end

if ~isfield(param, 'weights')
    % Neighbourhood
    param.weights = [];
end

if ~isfield(param, 'underrelax') || isempty(param.underrelax)
    % Underrelaxation (1 = no underrelaxation)
    param.underrelax = 1;
elseif param.underrelax > 1 || param.underrelax <= 0
    error('Underrelaxation parameter needs to be in (0,1]!')
end

if ~isfield(param, 'relax') || isempty(param.relax)
    % Social cosparsity relaxation
    param.relax = 0.9;
% elseif (param.relax > 1 || param.relax <= 0) 
%     error('Relaxation parameter needs to be in (0,1)!')
end
tStart = tic;
% Initial and static data
if strcmp(param.prior,'analysis')
    xHat = param.x0;
    zEst = A(xHat,'notransp');
else
    zEst = param.x0;
    xHat = y;
end

p = length(zEst);
blocks = size(y,2);

if ~isfield(param, 'iter') || isempty(param.iter)
    % Iteration threshold
    param.iter = p;
end

%%
if param.loud
    disp(['Running general TightSPADE:'])
    disp(['Accuracy = ' num2str(param.accuracy)])
    disp(['Iterations = ' num2str(param.iter)])
    disp(['Initial point: ' initPoint])
    disp('-----------------------------------------')
    disp(['i' 9 'object' 9 'constr' 9 'k'])
end

u = zeros(p,size(zEst,2));

bestObj = Inf;
%%
k = param.greedy;

% Shrinkage parameters
shrinkParam.isRedundant = param.isRedundant;
shrinkParam.prev_blocks = prev_blocks;
shrinkParam.W = param.weights;

% Projection parameters
proxParam.epsilon = param.noise;
proxParam.rel_ind = relChunk;
proxParam.clip_ind = ~relChunk;
proxParam.clip_pos = (proxParam.clip_ind & y>0);
proxParam.clip_neg = (proxParam.clip_ind & y<=0);

    

% Iterating ADMM procedure
for i=1:param.iter
    
    xOld = xHat;
    zOld = zEst;
    
    %% zHat update
    zThreshold = zEst + u;
    
    % Shrinkage (projection to the sparse constraint)
    zHat = shrink(zThreshold, k, shrinkParam);
    
    
        %% Evaluate the objective
    funVal = norm(zHat - zEst, 'fro')/(eps+norm(zHat,'fro'));
    
    param.objective(i) = funVal;
    
    if bestObj >= funVal
        bestObj = funVal;
    end
    
    if param.loud
        fprintf('%d\t%7f\t%d\n', i, funVal,k);
    end
        
    % Termination criteria
    if funVal < param.accuracy
        break
    end     


    %% Estimate update
    % projection to the data-fidelity constraint
    b = zHat - u;
    
    if(strcmp(param.prior,'analysis'))% analysis sparse
        [xHat, proxParam] = prox(b,y,A,proxParam);
        xHat = param.underrelax*xHat + (1-param.underrelax)*xOld;
        zEst = A(xHat,'notransp');
    else%synthesis sparse
        [zEst, proxParam] = prox(b,y,A,proxParam);
        zEst = param.underrelax*zEst + (1-param.underrelax)*zOld;
    end
    
    %% Compute the dual variable
    u = u - zHat + zEst;
    
    
    %% Set sparsity level
    if param.crime
        k = param.greedy;
    else
        if mod(i,param.rate) == 0
            k = updateSparsity(k,param.relax);
        end
    end
 
end

% Iteration log
if strcmpi(param.prior,'synthesis')
    xOut = real(A(zEst,'transp'));
else % analysis prior
    xOut = real(xHat);
end
zOut = zHat;
param.x0 = [];
param.regularizer = k;
param.time = toc(tStart);
param.iterCount = i;
varargout{1} = param;

if param.loud
    disp(['generalTightSPADE done! (t = ' num2str(param.time) 's)'])
end

end




