function  runAudioRestorationExperiment(CleanFile, SNR, param)
%% runAudioRestorationExperiment is the global wrapper for the sparse audio restoration experiments
%
%   Input:
%       CleanFile: string, with the clean file name
%       SNR: double, signal-to-noise / signal-to-distortion ratio
%       param: parameters structure
%
% ************************************************************
% This software is available under the  BSD-3-Clause License  (https://opensource.org/licenses/BSD-3-Clause)
% ************************************************************
% 3-Clause BSD License
% 
% Copyright <2016-2019><INRIA - CNRS><Clément Gaultier, Srdan Kitic, Ewen Camberlein, Rémi Gribonval, Nancy Bertin>
% 
% Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
% 
% (i) Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
% 
% (ii) Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
% 
% (iii) Neither the name of the copyright holder nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.
% 
% THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
% 
% Contact: nancy.bertin@irisa.fr, frederic.bimbot@irisa.fr, remi.gribonval@inria.fr
% PANAMA Research Group,  IRISA  -  (Inria & CNRS)
% Campus de Beaulieu, 35042 Rennes cedex, France
%


%% Setting paths

addpath('./../AudioProblem/','./../Measure/','./../Core/','./../Core/Projections/', './../Core/SparsityUpdates/',...
        './../Core/Shrinkages/','./../SignalTools/');
outputDIR = './../../Results/';
inputDIR = './../../CleanAudioData/';
fileNameOriginal = [inputDIR CleanFile];
FileInfo = audioinfo(fileNameOriginal);
[~,fileName,~] = fileparts(fileNameOriginal);

%% Parameters initialization

param.Fs = FileInfo.SampleRate;
param.Nbits = FileInfo.BitsPerSample;
param.NChan = FileInfo.NumChannels;
param.iSNR = SNR;            % input level of the degradation in dB (SNR for denoising/ SDR for declipping)
%param.window = @hanning;    % Default window is Hamming
param.w_size = pow2(nextpow2(param.window_length*param.Fs));         % Window size in samples 
param.overlap = 0.75;        % Overlap between frames in %
param.stepsize = param.w_size*(1-param.overlap);       % Overlap add stepsize (needs to satisfy COLA condition!)
if param.Fs > 16000
    param.iter = 1300;       % Maximum number of iterations (upper bound for the algorithm used with PEW shrinkage)
else
    param.iter = 1e6;
end
param.rate = 1;              % Relaxation rate (=1, we decrease lambda in each iteration)
param.greedy = 1;            % sparsity relaxation number for Hard Thresholding shrinkage
param.underrelax = 0.9;      % ADMM underrelaxation parameter (0,1]
param.causal = false;        % caching previous time blocks estimates in the social case (save 30% time but could affect performance see. ref [1] in Readme.txt)

%% Problem generation

switch upper(param.Pb)
    case 'DENOISING'
        if param.NChan > 1
            error('Denoising on multichannel not available for now.');
        else
            [DegradedData, CleanData, noiseVar] = generateDenoisingExp(CleanFile, SNR, param);
        end
    case 'DECLIPPING'
        [DegradedData, CleanData] = generateDeclippingExp(CleanFile, SNR, param);
    otherwise
        error('Unknown audio restoration problem!');
end


%% Processing and performance measure

for chan = 1:param.NChan
    disp(['Processing channel: ' num2str(chan)])
    [RestoredData(:,chan),param,Perf{chan}] = enhanceAllFrames(DegradedData(:,chan), CleanData(:,chan), param);
    OldSNR(chan) = getSDR(CleanData(:,chan), DegradedData(:,chan));
    NewSNR(chan) = getSDR(CleanData(:,chan), RestoredData(:,chan));
    Perf{chan}.SNRImprovement = round(NewSNR(chan)-OldSNR(chan),2);
    Perf{chan}.RTratio = round(Perf{chan}.TimeElapsed/FileInfo.Duration,2);
end

%% Reporting and saving 

switch upper(param.Pb)
    case 'DECLIPPING'
        degradedPATH = [outputDIR 'Redundancy' num2str(param.redundancy) '/Declipping/'...
            num2str(param.iSNR) '/'];
        degradedFile = [fileName '_' num2str(param.iSNR) 'dB_clip_input.wav'];
        nBitDeg = 64;
        enhancedPATH = [outputDIR 'Redundancy' num2str(param.redundancy) '/Declipping/'...
            num2str(param.iSNR) '/' param.prior '/' param.Shrinkage '/'];
        enhancedFile = [fileName '_' num2str(param.iSNR)...
            'dB_declip_' param.Shrinkage '.wav'];
        perfFile = [fileName '_' num2str(param.iSNR)...
            'dB_declip_' param.Shrinkage '.mat'];

    case 'DENOISING'

        degradedPATH = [outputDIR 'Redundancy' num2str(param.redundancy) '/Denoising/'...
            num2str(param.iSNR) '/'];
        degradedFile = [fileName '_' num2str(param.iSNR) 'dB_noisy_input.wav'];
        nBitDeg = param.Nbits;
        enhancedPATH = [outputDIR 'Redundancy' num2str(param.redundancy) '/Denoising/'...
            num2str(param.iSNR) '/' param.prior '/' param.Shrinkage '/'];
        enhancedFile = [fileName '_' num2str(param.iSNR)...
            'dB_denoise_' param.Shrinkage '.wav'];
        perfFile = [fileName '_' num2str(param.iSNR)...
            'dB_denoise_' param.Shrinkage '.mat'];

    otherwise
        error('Unknown audio restoration problem');
end

disp('Saving results...')

if ~isdir(degradedPATH)
    mkdir(degradedPATH);
end
if ~isdir(enhancedPATH)
    mkdir(enhancedPATH);
end

if ~exist([degradedPATH degradedFile],'file')
    audiowrite([degradedPATH degradedFile], DegradedData, param.Fs, 'BitsPerSample', nBitDeg);
end

audiowrite([enhancedPATH enhancedFile], RestoredData, param.Fs, 'BitsPerSample', param.Nbits);

disp(['New SNR: ' num2str(round(NewSNR,2)) 'dB; Old SNR: ' num2str(round(OldSNR,2)) 'dB'])
disp(['Improvement: ' num2str(round(NewSNR-OldSNR,2)) 'dB'])

save([enhancedPATH perfFile], 'NewSNR', 'OldSNR', 'Perf', 'param');


disp('Done!')


end

