function [best_w, best_greedy, xBest,alg] = selectTFPattern( yChunk, A, relChunk, prev_blocks,...
    max_y, prox, shrink, updateSparsity, alg )
%% selectTFPattern return the optimal TF neighborhood to be used for the
%audio reconstruction task based on the shanon entropy criterion in case of
%social modeling
%
%   Input:
%       yChunk: the imput signal to be tested
%       A: function handles of the frequency transforms
%       relChunk: logical vector or matrix with the reliable samples in
%                 case of declipping
%       prev_blocks: cached previous blocks in case of 'causal' option with
%                   social versions (see. AUDASCITY publication)
%       max_y: maximum absolute magnitude of yChunk
%       prox: function handle on the proximal operator used
%       shrink: function handle on the sparsifying operator used
%       updateSparsity: function handle on the update rule of the sparsity
%                       hyperparameter
%       alg: parameters structure
%
%   Output:
%       best_w: best time-frequency pattern
%       xBest: best estimate from y to warm start the rest of the procedure
%       best_greedy: best greedy param to warm start the rest
%       tElapsed: elapsed time
%       alg: updated parameters structure
%
% ************************************************************
% This software is available under the  BSD-3-Clause License  (https://opensource.org/licenses/BSD-3-Clause)
% ************************************************************
% 3-Clause BSD License
% 
% Copyright <2016-2019><INRIA - CNRS><Clément Gaultier, Srdan Kitic, Ewen Camberlein, Rémi Gribonval, Nancy Bertin>
% 
% Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
% 
% (i) Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
% 
% (ii) Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
% 
% (iii) Neither the name of the copyright holder nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.
% 
% THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
% 
% Contact: nancy.bertin@irisa.fr, frederic.bimbot@irisa.fr, remi.gribonval@inria.fr
% PANAMA Research Group,  IRISA  -  (Inria & CNRS)
% Campus de Beaulieu, 35042 Rennes cedex, France   


num_weights = numel(alg.Gamma);
beststencil = -Inf;
bestimprove = -Inf;
tElapsed=0;

if strcmpi(alg.Pb,'DECLIPPING') && strcmpi(alg.prior,'synthesis')
    alg.crime = true; % do not update the greedy param for the 10 iterations while chosing the neigborhood
end

for w = 1 : num_weights
    alg.time = 0;    
    alg.weights = alg.Gamma{w};
    % Initial value for lambda == (loose) upper bound of the neighbourhood
    switch upper(alg.Pb)
        case 'DECLIPPING'
            alg.greedy = (1-max_y)*nnz(alg.weights(:));
        case 'DENOISING'
            alg.greedy = max_y*nnz(alg.weights(:));
        otherwise
            error('Unknown degradation problem!')
    end
    
    
    [xHat, zHat, alg] = generalTightSPADE(yChunk, A, prox, shrink, updateSparsity, relChunk, prev_blocks,alg);
    
    tElapsed = tElapsed + alg.time;
    
    residual_new = abs(A(yChunk - xHat,'notransp'));
    
    residual = [alg.residual_old residual_new];
    % 1+ log2 : Herbert sturges law
    numbins = round(1+log2(numel(residual)));
    curmetric = getEntropy(residual(:),numbins);
    
    improve = getSDR(alg.True_f,xHat(:,alg.midblock)) - getSDR(alg.True_f,yChunk(:,alg.midblock));
    
    if curmetric > beststencil
        beststencil = curmetric;
        best_w = w;
        xBest = xHat;
        best_greedy = alg.regularizer;
        
    end
    
    if improve > bestimprove
        bestimprove = improve;
        optimal_w = w;
    end
    tElapsed = tElapsed + alg.time;
end


end

